import unittest
import pytest

# from selenium.webdriver.common.by import By
# from selenium.webdriver.common.keys import Keys
# from selenium.webdriver.support import expected_conditions as expect
from selenium.webdriver.support.ui import WebDriverWait
from datetime import datetime

from pastasauce import PastaSauce, PastaDecorator
from . import StaxHelper, Teacher
try:
    from . import Assignment
except:
    from staxing.assignment import Assignment

NOT_STARTED = True

browsers = [{
    "platform": "Windows 7",
    "browserName": "firefox",
    "version": "40.0",
    "screenResolution": "1440x900"
}]

standard_window = (1440, 800)
compressed_window = (700, 500)


@PastaDecorator.on_platforms(browsers)
class TestTutorAssignments(unittest.TestCase):
    ''''''
    def setUp(self):
        self.ps = PastaSauce()
        self.desired_capabilities['name'] = self.id()
        self.teacher = Teacher(use_env_vars=True)
        self.helper = StaxHelper(driver_type='chrome', pasta_user=self.ps,
                                 capabilities=self.desired_capabilities,
                                 initial_user=self.teacher)
        self.driver = self.helper.driver
        self.wait = WebDriverWait(self.driver, StaxHelper.DEFAULT_WAIT_TIME)
        self.driver.set_window_size(*standard_window)
        self.teacher.login(self.driver)
        self.teacher.select_course(self.driver, category='physics')
        self.rword = Assignment.rword
        self.screenshot_path = '/tmp/errors/'

    def tearDown(self):
        # Returns the info of exception being handled
        has_errors = self._test_has_failed()
        if has_errors:
            print(self.driver.current_url, '\n')
            date_and_time = datetime.now().strftime('%Y-%m-%d_%H-%M-%S-%f')
            filename = 'testerr_%s.png' % date_and_time
            self.driver.save_screenshot('%s%s' % (self.screenshot_path,
                                        filename))
        self.driver.quit()
        self.ps.update_job(self.driver.session_id, passed=has_errors)

    def _test_has_failed(self):
        # for 3.4. In 3.3, can just use self._outcomeForDoCleanups.success:
        for method, error in self._outcome.errors:
            if error:
                return True
        return False

    # @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_by_period_draft_to_all_cancel_to_period_delete(self):
        '''
        '''
        # 1st / 0 / 1 / 1S / Draft
        # 2nd / 0 / 1 / 1S
        # all / 0 / 1 / 1S / Cancel
        # 1st / 0 / 1 / 1S / Delete
        # 2nd / 0 / 1 / 1S
        name = 'Automated 2'
        description = 'by period draft to all cancel to period delete'
        open_p1 = StaxHelper.date_string()
        close_p1 = StaxHelper.date_string(1)
        open_p2 = StaxHelper.date_string(1)
        close_p2 = StaxHelper.date_string(2)
        open_all = StaxHelper.date_string()
        close_all = StaxHelper.date_string(1)
        try:
            self.assign.add_new_reading(
                driver=self.driver,
                title=name,
                description=description,
                periods={
                    '1st': (open_p1, close_p1),
                    '2nd': (open_p2, close_p2), },
                readings=['1.1'],
                status=self.assign.DRAFT, )
        except Exception as ex:
            self.fail('1: Add :: %s :: %s' % (ex.__class__.__name__, ex))
        try:
            self.assign.change_reading(
                driver=self.driver,
                title=name,
                periods={
                    'all': (open_all, close_all), },
                status=self.assign.CANCEL)
        except Exception as ex:
            self.fail('2: Edit :: %s :: %s' % (ex.__class__.__name__, ex))
        try:
            self.assign.remove_reading(
                driver=self.driver,
                title=name)
        except Exception as ex:
            self.fail('3: Remove :: %s :: %s' % (ex.__class__.__name__, ex))

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_by_period_draft_to_all_draft_to_delete(self):
        '''
        '''
        # 1st / 0 / 1 / 1S / Draft
        # 2nd / 1 / 2 / 1S
        # all / 0 / 2 / 1S / Draft
        # all / 0 / 2 / 1S / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_by_period_publish_to_all_publish_fail(self):
        '''
        '''
        # 1st / 0 / 1 / 1S / Publish
        # 2nd / 0 / 1 / 1S
        # all / 0 / 1 / 1S / Publish **

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_by_period_publish_to_delete_fail(self):
        '''
        '''
        # 1st / 0 / 1 / 1S / Publish
        # 2nd / 1 / 2 / 1S
        # 1st / 0 / 1 / 1S / Delete **
        # 2nd / 1 / 2 / 1S

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_by_period_publish_to_all_publish_to_delete_fail(self):
        '''
        '''
        # 1st / 0 / 1 / 1S / Publish
        # 2nd / 1 / 2 / 1S
        # all / 0 / 2 / 1S / Publish
        # all / 0 / 2 / 1S / Delete **

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_by_period_publish_to_all_cancel_to_period_delete(self):
        '''
        '''
        # 1st / 1 / 2 / 1S / Publish
        # 2nd / 1 / 2 / 1S
        # all / 1 / 2 / 1S / Cancel
        # 1st / 1 / 2 / 1S / Delete
        # 2nd / 1 / 2 / 1S

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_status_by_period_publish_to_all_publish_to_delete(
            self):
        '''
        '''
        # 1st / 1 / 2 / 1S / Publish
        # 2nd / 1 / 2 / 1S
        # all / 1 / 2 / 1S / Publish
        # all / 1 / 2 / 1S / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_by_period_publish_to_delete(self):
        '''
        '''
        # 1st / 1 / 2 / 1S / Publish
        # 2nd / 2 / 3 / 1S
        # 1st / 1 / 2 / 1S / Delete
        # 2nd / 2 / 3 / 1S

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_dates_by_period_publish_to_all_publish_to_delete(self):
        '''
        '''
        # 1st / 1 / 2 / 1S / Publish
        # 2nd / 2 / 3 / 1S
        # all / 1 / 3 / 1S / Publish
        # all / 1 / 3 / 1S / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_all_cancel(self):
        '''
        '''
        # all / 0 / 1 / 1S / Cancel

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_all_draft_to_period_cancel_to_all_delete(self):
        '''
        '''
        # all / 0 / 1 / 1S / Draft
        # 1st / 0 / 1 / 1S / Cancel
        # 2nd / 0 / 1 / 1S
        # all / 0 / 1 / 1S / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_all_draft_to_period_draft_to_delete(self):
        '''
        '''
        # all / 0 / 1 / 1S / Draft
        # 1st / 0 / 1 / 1S / Draft
        # 2nd / 1 / 2 / 1S
        # 1st / 0 / 1 / 1S / Delete
        # 2nd / 1 / 2 / 1S

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_section_all_draft_to_delete(self):
        '''
        '''
        # all / 0 / 1 / 1S / Draft
        # all / 0 / 1 / 1S / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_chapter_all_draft_to_delete(self):
        '''
        '''
        # all / 1 / 2 / 1C / Draft
        # all / 1 / 2 / 1C / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_all_publish_to_period_publish_to_period_delete_fail(
            self):
        '''
        '''
        # all / 0 / 1 / 1S / Publish
        # 1st / 0 / 1 / 1S / Publish
        # 2nd / 0 / 1 / 1S
        # 1st / 0 / 1 / 1S / Delete **
        # 2nd / 0 / 1 / 1S

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_all_publish_to_period_publish_fail(self):
        '''
        '''
        # all / 0 / 1 / 1S / Publish
        # 1st / 0 / 1 / 1S / Publish **
        # 2nd / 0 / 1 / 1S

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_all_publish_to_delete_fail(self):
        '''
        '''
        # all / 0 / 1 / 1S / Publish
        # all / 0 / 1 / 1S / Delete **

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_due_dates_all_draft_to_cancel_to_delete(self):
        '''
        '''
        # all / 0 / 2 / 1S / Draft
        # all / 0 / 1 / 1S / Cancel
        # all / 0 / 2 / 1S / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_open_dates_all_draft_to_cancel_to_delete(self):
        '''
        '''
        # all / 1 / 2 / 1S / Draft
        # all / 0 / 2 / 1S / Cancel
        # all / 1 / 2 / 1S / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_dates_all_draft_to_cancel_to_delete(self):
        '''
        '''
        # all / 1 / 2 / 1S / Draft
        # all / 2 / 3 / 1S / Cancel
        # all / 1 / 2 / 1S / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_due_dates_all_draft_to_draft_to_delete(self):
        '''
        '''
        # all / 0 / 2 / 1S / Draft
        # all / 0 / 1 / 1S / Draft
        # all / 0 / 1 / 1S / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_due_earlier_all_publish_to_publish_to_delete_fail(
            self):
        '''
        '''
        # all / 0 / 2 / 1S / Publish
        # all / 0 / 1 / 1S / Publish
        # all / 0 / 1 / 1S / Delete **

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_due_later_all_publish_to_publish_to_delete_fail(self):
        '''
        '''
        # all / 0 / 2 / 1S / Publish
        # all / 0 / 3 / 1S / Publish
        # all / 0 / 3 / 1S / Delete **

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_sections_all_draft_to_draft_to_delete(self):
        '''
        '''
        # all / 1 / 2 / 1C  / Draft
        # all / 1 / 2 / 1C - 1S / Draft
        # all / 1 / 2 / 1C - 1S / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_all_draft_to_draft_to_publish_to_delete(
            self):
        '''
        '''
        # all / 1 / 2 / 1C  / Draft
        # all / 1 / 2 / 1C - 1S / Draft
        # all / 1 / 2 / 1C - 1S / Publish
        # all / 1 / 2 / 1C - 1S / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_chapter_plus_sections_all_draft_to_draft_to_delete(
            self):
        '''
        '''
        # all / 1 / 2 / 1C  / Draft
        # all / 1 / 2 / 1C + 1S / Draft
        # all / 1 / 2 / 1C + 1S / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_section_plus_sections_all_draft_to_draft_to_delete(
            self):
        '''
        '''
        # all / 1 / 2 / 1S  / Draft
        # all / 1 / 2 / 1S + 1S / Draft
        # all / 1 / 2 / 2S  / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_chapter_plus_sections_all_draft_to_publish_to_delete(
            self):
        '''
        '''
        # all / 1 / 2 / 1C  / Draft
        # all / 1 / 2 / 1C + 1S / Publish
        # all / 1 / 2 / 1C + 1S / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_section_plus_sections_all_draft_to_publish_to_delete(
            self):
        '''
        '''
        # all / 1 / 2 / 1S  / Draft
        # all / 1 / 2 / 1S + 1S / Publish
        # all / 1 / 2 / 2S  / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_chapter_minus_sections_all_publish_to_draft_to_delete(
            self):
        '''
        '''
        # all / 1 / 2 / 1C  / Publish
        # all / 1 / 2 / 1C - 1S / Draft
        # all / 1 / 2 / 1C - 1S / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_open_dates_all_publish_to_draft_to_delete(self):
        '''
        '''
        # all / 2 / 3 / 1S / Publish
        # all / 1 / 3 / 1S / Draft
        # all / 1 / 3 / 1S / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_section_minus_sections_all_publish_to_draft_to_delete(
            self):
        '''
        '''
        # all / 1 / 2 / 2S  / Publish
        # all / 1 / 2 / 2S - 1S / Draft
        # all / 1 / 2 / 1S  / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_ch_minus_sections_all_publish_to_publish_to_delete(
            self):
        '''
        '''
        # all / 1 / 2 / 1C  / Publish
        # all / 1 / 2 / 1C - 1S / Publish
        # all / 1 / 2 / 1C - 1S / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_dates_all_publish_to_publish_to_delete(self):
        '''
        '''
        # all / 1 / 2 / 1S / Publish
        # all / 2 / 3 / 1S / Publish
        # all / 2 / 3 / 1S / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_sec_minus_sections_all_publish_to_publish_to_delete(
            self):
        '''
        '''
        # all / 1 / 2 / 2S  / Publish
        # all / 1 / 2 / 2S - 1S / Publish
        # all / 1 / 2 / 1S  / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_chapter_plus_sections_all_publish_to_draft_to_delete(
            self):
        '''
        '''
        # all / 1 / 2 / 1C  / Publish
        # all / 1 / 2 / 1C + 1S / Draft
        # all / 1 / 2 / 1C + 1S / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_section_plus_sections_all_publish_to_draft_to_delete(
            self):
        '''
        '''
        # all / 1 / 2 / 1S  / Publish
        # all / 1 / 2 / 1S + 1S / Draft
        # all / 1 / 2 / 2S  / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_chapter_plus_sections_all_publish_to_publish_to_delete(
            self):
        '''
        '''
        # all / 1 / 2 / 1C  / Publish
        # all / 1 / 2 / 1C + 1S / Publish
        # all / 1 / 2 / 1C + 1S / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_chapter_all_draft_to_publish_to_delete(self):
        '''
        '''
        # all / 1 / 2 / 1C / Draft
        # all / 1 / 2 / 1C / Publish
        # all / 1 / 2 / 1C / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_chapter_all_publish_to_draft_to_delete(self):
        '''
        '''
        # all / 1 / 2 / 1C / Publish
        # all / 1 / 2 / 1C / Draft
        # all / 1 / 2 / 1C / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_chapter_all_publish_to_publish_to_delete(self):
        '''
        '''
        # all / 1 / 2 / 1C / Publish
        # all / 1 / 2 / 1C / Publish
        # all / 1 / 2 / 1C / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_section_plus_sections_all_publish_to_publish_to_delete(
            self):
        '''
        '''
        # all / 1 / 2 / 1S  / Publish
        # all / 1 / 2 / 1S + 1S / Publish
        # all / 1 / 2 / 2S  / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_open_dates_all_draft_to_draft_to_delete(self):
        '''
        '''
        # all / 1 / 2 / 1S / Draft
        # all / 0 / 2 / 1S / Draft
        # all / 0 / 2 / 1S / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_minus_sections_all_draft_to_draft_to_delete(self):
        '''
        '''
        # all / 1 / 2 / 2S  / Draft
        # all / 1 / 2 / 2S - 1S / Draft
        # all / 1 / 2 / 1S  / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_dates_all_draft_to_draft_to_delete(self):
        '''
        '''
        # all / 1 / 2 / 1S / Draft
        # all / 2 / 3 / 1S / Draft
        # all / 2 / 3 / 1S / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_all_draft_to_delete(self):
        '''
        '''
        # all / 1 / 2 / 1S / Draft
        # all / 1 / 2 / 1S / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_all_publish_to_period_cancel_to_all_delete(self):
        '''
        '''
        # all / 1 / 2 / 1S / Publish
        # 1st / 1 / 2 / 1S / Cancel
        # 2nd / 1 / 2 / 1S
        # all / 1 / 2 / 1S / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_all_publish_to_period_publish_to_delete(self):
        '''
        '''
        # all / 1 / 2 / 1S / Publish
        # 1st / 1 / 2 / 1S / Publish
        # 2nd / 1 / 2 / 1S
        # 1st / 1 / 2 / 1S / Delete
        # 2nd / 1 / 2 / 1S

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_all_publish_to_publish_to_delete_fail(self):
        '''
        '''
        # all / 1 / 2 / 1S / Publish
        # all / 0 / 2 / 1S / Publish
        # all / 0 / 2 / 1S / Delete **

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_all_publish_to_delete(self):
        '''
        '''
        # all / 1 / 2 / 1S / Publish
        # all / 1 / 2 / 1S / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_minus_section_all_draft_to_publish_to_delete(self):
        '''
        '''
        # all / 1 / 2 / 2S  / Draft
        # all / 1 / 2 / 2S - 1S / Publish
        # all / 1 / 2 / 1S  / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_dates_all_draft_to_publish_to_delete(self):
        '''
        '''
        # all / 2 / 3 / 1S / Draft
        # all / 1 / 3 / 1S / Publish
        # all / 1 / 3 / 1S / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_all_draft_to_draft_to_draft_to_delete(self):
        '''
        '''
        # all / 1 / 2 / 2S  / Draft
        # all / 1 / 2 / 2S + 1S / Draft
        # all / 1 / 2 / 3S - 1S / Draft
        # all / 1 / 2 / 2S  / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_all_draft_to_publish_to_draft_to_publish_to_delete(
            self):
        '''
        '''
        # all / 1 / 2 / 2S  / Draft
        # all / 1 / 2 / 2S + 1S / Publish
        # all / 1 / 2 / 3S - 1S / Draft
        # all / 1 / 2 / 2S  / Publish
        # all / 1 / 2 / 2S  / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_all_publish_to_draft_to_draft_to_delete(self):
        '''
        '''
        # all / 1 / 2 / 2S  / Publish
        # all / 1 / 2 / 2S + 1S / Draft
        # all / 1 / 2 / 3S - 1S / Draft
        # all / 1 / 2 / 2S  / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_all_publish_to_publish_to_publish_to_delete(self):
        '''
        '''
        # all / 1 / 2 / 2S  / Publish
        # all / 1 / 2 / 2S + 1S / Publish
        # all / 1 / 2 / 3S - 1S / Publish
        # all / 1 / 2 / 2S  / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_all_draft_to_draft_to_delete(self):
        '''
        '''
        # all / 2 / 3 / 1S / Draft
        # all / 1 / 3 / 1S / Draft
        # all / 1 / 3 / 1S / Delete

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_all_publish_to_publish_to_delete(self):
        '''
        '''
        # all / 2 / 3 / 1S / Publish
        # all / 1 / 3 / 1S / Publish
        # all / 1 / 3 / 1S / Delete



import unittest
import pytest

from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.support import expected_conditions as expect
from selenium.webdriver.support.ui import Select, WebDriverWait
from selenium.common.exceptions import NoSuchElementException
from datetime import datetime
from time import sleep

from pastasauce import PastaSauce, PastaDecorator
from . import StaxHelper, Admin
try:
    from . import Assignment
except:
    from staxing.assignment import Assignment

NOT_STARTED = True

browsers = [{
    "platform": "Windows 7",
    "browserName": "firefox",
    "version": "40.0",
    "screenResolution": "1440x900"
}]

standard_window = (1440, 800)
compressed_window = (700, 500)


@PastaDecorator.on_platforms(browsers)
class TestTutorAdmin(unittest.TestCase):
    def setUp(self):
        self.ps = PastaSauce()
        self.desired_capabilities['name'] = self.id()
        self.admin = Admin(use_env_vars=True)
        self.helper = StaxHelper(driver_type='chrome', pasta_user=self.ps,
                                 capabilities=self.desired_capabilities,
                                 initial_user=self.admin)
        self.driver = self.helper.driver
        self.wait = WebDriverWait(self.driver, StaxHelper.DEFAULT_WAIT_TIME)
        self.driver.set_window_size(*standard_window)
        self.admin.login(self.driver)
        self.rword = Assignment.rword
        self.screenshot_path = '/tmp/errors/'

    def tearDown(self):
        # Returns the info of exception being handled
        has_errors = self._test_has_failed()
        if has_errors:
            print(self.driver.current_url, '\n')
            date_and_time = datetime.now().strftime('%Y-%m-%d_%H-%M-%S-%f')
            filename = 'testerr_%s.png' % date_and_time
            self.driver.save_screenshot('%s%s' % (self.screenshot_path,
                                        filename))
        self.driver.quit()
        self.ps.update_job(self.driver.session_id, passed=has_errors)

    def _test_has_failed(self):
        # for 3.4. In 3.3, can just use self._outcomeForDoCleanups.success:
        for method, error in self._outcome.errors:
            if error:
                return True
        return False

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_admin_imports_a_new_ecosystem_book(self):
        self.admin.goto_admin_control(self.driver)
        self.admin.goto_ecosystems(self.driver)
        assert(self.driver.find_element(By.XPATH,
               '//h1[text()="Ecosystems"]')), 'Ecosystems not found'
        table = self.driver.find_element(By.ID, 'main')
        rows = table.find_elements(By.XPATH, './table/tbody/tr')
        assert(len(rows) >= 1), 'No previous ecosystem found'
        cells = rows[-1].find_elements(By.TAG_NAME, 'td')
        assert(len(cells) >= 1), 'No table cells available'
        uuid = cells[-1].find_element(By.TAG_NAME, 'a'). \
            get_attribute('data-content')
        self.wait.until(
            expect.visibility_of_element_located(
                (By.LINK_TEXT, 'Import a new Ecosystem')
            )
        ).click()
        self.wait.until(
            expect.visibility_of_element_located(
                (By.XPATH, '//h1[text()="Import a new Ecosystem"]')
            )
        )
        self.driver.find_element(By.ID, 'cnx_id'). \
            send_keys(uuid)
        self.driver.find_element(By.ID, 'comments'). \
            send_keys('Auto-test created')
        self.driver.find_element(By.XPATH, '//button[text()="Import"]').click()
        try:
            self.wait.until(
                expect.visibility_of_element_located(
                    (By.XPATH, '//div[contains(@class,"alert-info")]')
                )
            )
        except Exception as ex:
            self.fail('Import not queued :: %s :: %s' % (ex.__class__.__name__,
                      ex))

    # @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_admin_creates_a_district(self):
        self.admin.goto_admin_control(self.driver)
        self.wait.until(
            expect.element_to_be_clickable(
                (By.XPATH, '//a[contains(.,"Organization")]')
            )
        ).click()
        self.wait.until(
            expect.element_to_be_clickable(
                (By.LINK_TEXT, 'Districts')
            )
        ).click()
        try:
            self.wait.until(
                expect.visibility_of_element_located(
                    (By.XPATH, '//h1[text()="Manage districts"]')
                )
            )
        except Exception as ex:
            self.fail('District management page not loaded :: %s :: %s' %
                      (ex.__class__.__name__, ex))
        self.driver.find_element(By.LINK_TEXT, 'Add district').click()
        try:
            self.wait.until(
                expect.visibility_of_element_located(
                    (By.XPATH, '//h1[text()="Create a district"]')
                )
            )
        except Exception as ex:
            self.fail('District creation page not loaded :: %s :: %s' %
                      (ex.__class__.__name__, ex))
        self.driver.find_element(By.ID, 'district_name'). \
            send_keys('Auto-test District %s' % self.rword(4))
        self.driver.find_element(By.XPATH, '//input[@name="commit"]').click()
        try:
            self.wait.until(
                expect.visibility_of_element_located(
                    (By.XPATH, '//div[contains(@class,"alert-info")]')
                )
            )
        except Exception as ex:
            self.fail('District not created :: %s :: %s' %
                      (ex.__class__.__name__, ex))

    # @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_admin_edits_a_district(self):
        self.admin.goto_admin_control(self.driver)
        self.wait.until(
            expect.element_to_be_clickable(
                (By.XPATH, '//a[contains(text(),"Organization")]')
            )
        ).click()
        self.wait.until(
            expect.element_to_be_clickable(
                (By.LINK_TEXT, 'Districts')
            )
        ).click()
        try:
            self.wait.until(
                expect.visibility_of_element_located(
                    (By.XPATH, '//h1[text()="Manage districts"]')
                )
            )
        except Exception as ex:
            self.fail('District management page not loaded :: %s :: %s' %
                      (ex.__class__.__name__, ex))
        try:
            self.driver.find_element(
                By.XPATH,
                '//td[contains(text(),"Auto-test")]' +
                '/following-sibling::td/a[text()="edit"]'
            ).click()
        except NoSuchElementException as ex:
            self.fail('No Auto-test element :: %s :: %s' %
                      (ex.__class__.__name__, ex))
        try:
            self.wait.until(
                expect.visibility_of_element_located(
                    (By.XPATH, '//h1[text()="Edit district"]')
                )
            )
        except Exception as ex:
            self.fail('District edit page not loaded :: %s :: %s' %
                      (ex.__class__.__name__, ex))
        old = self.driver.find_element(By.ID, 'district_name'). \
            get_attribute('value')
        current = old[:-4] + self.rword(4)
        self.driver.find_element(By.ID, 'district_name').clear()
        self.driver.find_element(By.ID, 'district_name'). \
            send_keys(current)
        self.driver.find_element(By.XPATH, '//input[@name="commit"]').click()
        try:
            self.wait.until(
                expect.visibility_of_element_located(
                    (By.XPATH, '//div[contains(@class,"alert-info")]')
                )
            )
        except Exception as ex:
            self.fail('District not updated :: %s :: %s' %
                      (ex.__class__.__name__, ex))

    # @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_admin_deletes_a_district(self):
        self.admin.goto_admin_control(self.driver)
        self.wait.until(
            expect.element_to_be_clickable(
                (By.XPATH, '//a[contains(text(),"Organization")]')
            )
        ).click()
        self.wait.until(
            expect.element_to_be_clickable(
                (By.LINK_TEXT, 'Districts')
            )
        ).click()
        try:
            self.wait.until(
                expect.visibility_of_element_located(
                    (By.XPATH, '//h1[text()="Manage districts"]')
                )
            )
        except Exception as ex:
            self.fail('District management page not loaded :: %s :: %s' %
                      (ex.__class__.__name__, ex))
        try:
            self.driver.find_element(
                By.XPATH,
                '//td[contains(text(),"Auto-test")]' +
                '/following-sibling::td/a[text()="delete"]'
            ).click()
        except Exception as ex:
            self.fail('No Auto-test district element to remove :: %s :: %s' %
                      (ex.__class__.__name__, ex))
        self.driver.switch_to_alert().accept()
        try:
            self.wait.until(
                expect.visibility_of_element_located(
                    (By.XPATH, '//div[contains(@class,"alert-info")]')
                )
            )
        except Exception as ex:
            self.fail('District not deleted :: %s :: %s' %
                      (ex.__class__.__name__, ex))

    # @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_admin_creates_a_school(self):
        ''''''
        self.admin.goto_admin_control(self.driver)
        self.wait.until(
            expect.element_to_be_clickable(
                (By.XPATH, '//a[contains(text(),"Organization")]')
            )
        ).click()
        self.wait.until(
            expect.element_to_be_clickable(
                (By.LINK_TEXT, 'Schools')
            )
        ).click()
        try:
            self.wait.until(
                expect.visibility_of_element_located(
                    (By.XPATH, '//h1[text()="Manage schools"]')
                )
            )
        except Exception as ex:
            self.fail('School management page not loaded :: %s :: %s' %
                      (ex.__class__.__name__, ex))
        self.driver.find_element(By.LINK_TEXT, 'Add school').click()
        try:
            self.wait.until(
                expect.visibility_of_element_located(
                    (By.XPATH, '//h1[text()="Create a school"]')
                )
            )
        except Exception as ex:
            self.fail('School creation page not loaded :: %s :: %s' %
                      (ex.__class__.__name__, ex))
        self.driver.find_element(By.ID, 'school_name'). \
            send_keys('Auto-test School %s' % self.rword(4))
        Select(self.driver.find_element(By.TAG_NAME, 'select')). \
            select_by_visible_text('OpenStax')
        self.driver.find_element(By.XPATH, '//input[@name="commit"]').click()
        try:
            self.wait.until(
                expect.visibility_of_element_located(
                    (By.XPATH, '//div[contains(@class,"alert-info")]')
                )
            )
        except Exception as ex:
            self.fail('School not created :: %s :: %s' %
                      (ex.__class__.__name__, ex))

    # @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_admin_edits_a_school(self):
        ''''''
        self.admin.goto_admin_control(self.driver)
        self.wait.until(
            expect.element_to_be_clickable(
                (By.XPATH, '//a[contains(text(),"Organization")]')
            )
        ).click()
        self.wait.until(
            expect.element_to_be_clickable(
                (By.LINK_TEXT, 'Schools')
            )
        ).click()
        try:
            self.wait.until(
                expect.visibility_of_element_located(
                    (By.XPATH, '//h1[text()="Manage schools"]')
                )
            )
        except Exception as ex:
            self.fail('School management page not loaded :: %s :: %s' %
                      (ex.__class__.__name__, ex))
        try:
            self.driver.find_element(
                By.XPATH,
                '//td[contains(text(),"Auto-test")]' +
                '/following-sibling::td/a[text()="edit"]'
            ).click()
        except NoSuchElementException as ex:
            self.fail('No Auto-test element :: %s :: %s' %
                      (ex.__class__.__name__, ex))
        try:
            self.wait.until(
                expect.visibility_of_element_located(
                    (By.XPATH, '//h1[text()="Edit school"]')
                )
            )
        except Exception as ex:
            self.fail('School edit page not loaded :: %s :: %s' %
                      (ex.__class__.__name__, ex))
        old = self.driver.find_element(By.ID, 'school_name'). \
            get_attribute('value')
        current = old[:-4] + self.rword(4)
        self.driver.find_element(By.ID, 'school_name').clear()
        self.driver.find_element(By.ID, 'school_name'). \
            send_keys(current)
        Select(self.driver.find_element(By.TAG_NAME, 'select')). \
            select_by_visible_text('OpenStax')
        self.driver.find_element(By.XPATH, '//input[@name="commit"]').click()
        try:
            self.wait.until(
                expect.visibility_of_element_located(
                    (By.XPATH, '//div[contains(@class,"alert-info")]')
                )
            )
        except Exception as ex:
            self.fail('School not updated :: %s :: %s' %
                      (ex.__class__.__name__, ex))

    # @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_admin_deletes_a_school(self):
        ''''''
        self.admin.goto_admin_control(self.driver)
        self.wait.until(
            expect.element_to_be_clickable(
                (By.XPATH, '//a[contains(text(),"Organization")]')
            )
        ).click()
        self.wait.until(
            expect.element_to_be_clickable(
                (By.LINK_TEXT, 'Schools')
            )
        ).click()
        try:
            self.wait.until(
                expect.visibility_of_element_located(
                    (By.XPATH, '//h1[text()="Manage schools"]')
                )
            )
        except Exception as ex:
            self.fail('School management page not loaded :: %s :: %s' %
                      (ex.__class__.__name__, ex))
        self.driver.find_element(
            By.XPATH,
            '//td[contains(text(),"Auto-test")]' +
            '/following-sibling::td/a[text()="delete"]'
        ).click()
        self.driver.switch_to_alert().accept()
        try:
            self.wait.until(
                expect.visibility_of_element_located(
                    (By.XPATH, '//div[contains(@class,"alert-info")]')
                )
            )
        except Exception as ex:
            self.fail('School not deleted :: %s :: %s' %
                      (ex.__class__.__name__, ex))

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_admin_creates_a_course(self):
        ''''''
        self.admin.goto_admin_control(self.driver)
        self.wait.until(
            expect.element_to_be_clickable(
                (By.XPATH, '//a[contains(text(),"Organization")]')
            )
        ).click()
        self.wait.until(
            expect.element_to_be_clickable(
                (By.LINK_TEXT, 'Courses')
            )
        ).click()
        try:
            self.wait.until(
                expect.visibility_of_element_located(
                    (By.XPATH, '//h1[text()="Courses"]')
                )
            )
        except Exception as ex:
            self.fail('Course management page not loaded :: %s :: %s' %
                      (ex.__class__.__name__, ex))
        self.driver.find_element(By.LINK_TEXT, 'Add Course').click()
        try:
            self.wait.until(
                expect.visibility_of_element_located(
                    (By.XPATH, '//h1[text()="New Course"]')
                )
            )
        except Exception as ex:
            self.fail('Course creation page not loaded :: %s :: %s' %
                      (ex.__class__.__name__, ex))
        self.driver.find_element(By.ID, 'course_name'). \
            send_keys('Auto-test Course %s' % self.rword(4))
        Select(self.driver.find_element(By.TAG_NAME, 'select')). \
            select_by_visible_text('OpenStax Ed')
        self.driver.find_element(By.XPATH, '//input[@name="commit"]').click()
        try:
            self.wait.until(
                expect.visibility_of_element_located(
                    (By.XPATH, '//div[contains(@class,"alert-info")]')
                )
            )
        except Exception as ex:
            self.fail('Course not created :: %s :: %s' %
                      (ex.__class__.__name__, ex))

    # @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_admin_edits_a_course(self):
        ''''''
        self.admin.goto_admin_control(self.driver)
        self.wait.until(
            expect.element_to_be_clickable(
                (By.XPATH, '//a[contains(text(),"Organization")]')
            )
        ).click()
        self.wait.until(
            expect.element_to_be_clickable(
                (By.LINK_TEXT, 'Courses')
            )
        ).click()
        try:
            self.wait.until(
                expect.visibility_of_element_located(
                    (By.XPATH, '//h1[text()="Courses"]')
                )
            )
        except Exception as ex:
            self.fail('Course management page not loaded :: %s :: %s' %
                      (ex.__class__.__name__, ex))
        try:
            self.driver.find_element(
                By.XPATH,
                '//td[contains(text(),"Auto-test")]' +
                '/following-sibling::td/a[text()="Edit"]'
            ).click()
        except NoSuchElementException as ex:
            self.fail('No Auto-test element :: %s :: %s' %
                      (ex.__class__.__name__, ex))
        old = self.driver.find_element(By.ID, 'course_name'). \
            get_attribute('value')
        current = old[:-4] + self.rword(4)
        self.driver.find_element(By.ID, 'course_name').clear()
        self.driver.find_element(By.ID, 'course_name'). \
            send_keys(current)
        Select(self.driver.find_element(By.TAG_NAME, 'select')). \
            select_by_visible_text('OpenStax Ed')
        self.driver.find_element(By.XPATH, '//input[@name="commit"]').click()
        try:
            self.wait.until(
                expect.visibility_of_element_located(
                    (By.XPATH, '//div[contains(@class,"alert-info")]')
                )
            )
        except Exception as ex:
            self.fail('Course name not changed :: %s :: %s' %
                      (ex.__class__.__name__, ex))
        try:
            self.driver.find_element(
                By.XPATH,
                '//td[contains(text(),"Auto-test")]' +
                '/following-sibling::td/a[text()="Edit"]'
            ).click()
        except Exception as ex:
            self.fail('No Auto-test element :: %s :: %s' %
                      (ex.__class__.__name__, ex))
        self.wait.until(
            expect.element_to_be_clickable(
                (By.XPATH, '//a[contains(text(),"Teachers")]')
            )
        ).click()
        try:
            self.wait.until(
                expect.visibility_of_element_located(
                    (By.XPATH, '//h3[contains(text(),"Assign teachers")]')
                )
            )
        except Exception as ex:
            self.fail('Assign Teachers page not loaded :: %s :: %s' %
                      (ex.__class__.__name__, ex))
        try:
            self.driver.find_element(By.XPATH, '//td[text()="demo_teacher"]')
        except:
            sleep(0.5)
            ActionChains(self.driver). \
                move_to_element(self.driver.find_element(
                    By.ID, 'course_teacher')). \
                click(). \
                send_keys('demo_teacher'). \
                send_keys(Keys.ARROW_DOWN). \
                perform()
            sleep(0.5)
            ActionChains(self.driver). \
                move_to_element(self.driver.find_element(
                    By.XPATH, '//ul[contains(@class,"ui-autocomplete")]')). \
                click(). \
                perform()
            try:
                self.wait.until(
                    expect.visibility_of_element_located(
                        (By.XPATH, '//div[contains(@class,"alert-info")]')
                    )
                )
            except Exception as ex:
                self.fail('Teacher not set :: %s :: %s' %
                          (ex.__class__.__name__, ex))
        self.wait.until(
            expect.element_to_be_clickable(
                (By.PARTIAL_LINK_TEXT, 'Course content')
            )
        ).click()
        try:
            self.wait.until(
                expect.visibility_of_element_located(
                    (By.XPATH, '//h3[text()="Select a course ecosystem"]')
                )
            )
        except Exception as ex:
            self.fail('Select Ecosystem page not loaded :: %s :: %s' %
                      (ex.__class__.__name__, ex))
        self.wait.until(
            expect.element_to_be_clickable(
                (By.PARTIAL_LINK_TEXT, 'Periods')
            )
        ).click()
        try:
            self.wait.until(
                expect.visibility_of_element_located(
                    (By.XPATH, '//h3[text()="Periods"]')
                )
            )
        except Exception as ex:
            self.fail('Periods page not loaded :: %s :: %s' %
                      (ex.__class__.__name__, ex))
        self.wait.until(
            expect.element_to_be_clickable(
                (By.PARTIAL_LINK_TEXT, 'Student Roster')
            )
        ).click()
        try:
            self.wait.until(
                expect.visibility_of_element_located(
                    (By.XPATH, '//h3[text()="Upload Student Roster"]')
                )
            )
        except Exception as ex:
            self.fail('Student Roster upload page not loaded :: %s :: %s' %
                      (ex.__class__.__name__, ex))
        self.driver.find_element(By.LINK_TEXT, 'Main Dashboard').click()
        self.admin.goto_admin_control(self.driver)
        self.wait.until(
            expect.element_to_be_clickable(
                (By.XPATH, '//a[contains(text(),"Organization")]')
            )
        ).click()
        self.wait.until(
            expect.element_to_be_clickable(
                (By.LINK_TEXT, 'Courses')
            )
        ).click()
        try:
            self.wait.until(
                expect.visibility_of_element_located(
                    (By.XPATH, '//h1[text()="Courses"]')
                )
            )
        except Exception as ex:
            self.fail('Course management page not loaded :: %s :: %s' %
                      (ex.__class__.__name__, ex))

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_admin_adds_a_teacher_to_a_course(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_admin_removes_a_teacher_from_a_course(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_admin_add_period_to_a_course(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_admin_changes_period_name(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_admin_creates_a_new_user_account(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_admin_adds_student_to_period(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_admin_removes_student_from_a_course(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_admin_moves_student_to_another_period(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_admin_impersonates_a_teacher(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_admin_impersonates_a_student(self):
        ''''''



import unittest
# import pytest

from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support import expected_conditions as expect
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.common.action_chains import ActionChains
from datetime import datetime

from pastasauce import PastaSauce, PastaDecorator
from . import StaxHelper, Teacher
try:
    from . import Assignment
except:
    from staxing.assignment import Assignment

# NOT_STARTED = True

browsers = [{
    "platform": "Windows 7",
    "browserName": "firefox",
    "version": "40.0",
    "screenResolution": "1440x900"
}]

standard_window = (1440, 800)
compressed_window = (700, 500)


@PastaDecorator.on_platforms(browsers)
class TestTutorAccountManagement(unittest.TestCase):
    ''''''
    def setUp(self):
        self.ps = PastaSauce()
        self.desired_capabilities['name'] = self.id()
        self.teacher = Teacher(use_env_vars=True)
        self.helper = StaxHelper(driver_type='chrome', pasta_user=self.ps,
                                 capabilities=self.desired_capabilities,
                                 initial_user=self.teacher)
        self.driver = self.helper.driver
        self.wait = WebDriverWait(self.driver, StaxHelper.DEFAULT_WAIT_TIME)
        self.driver.set_window_size(*standard_window)
        # self.teacher.login(self.driver)
        # self.teacher.select_course(self.driver, category='physics')
        self.rword = Assignment.rword
        self.screenshot_path = '/tmp/errors/'

    def tearDown(self):
        # Returns the info of exception being handled
        has_errors = self._test_has_failed()
        if has_errors:
            print(self.driver.current_url, '\n')
            date_and_time = datetime.now().strftime('%Y-%m-%d_%H-%M-%S-%f')
            filename = 'testerr_%s.png' % date_and_time
            self.driver.save_screenshot('%s%s' % (self.screenshot_path,
                                        filename))
        self.driver.quit()
        self.ps.update_job(self.driver.session_id, passed=has_errors)

    def _test_has_failed(self):
        # for 3.4. In 3.3, can just use self._outcomeForDoCleanups.success:
        for method, error in self._outcome.errors:
            if error:
                return True
        return False

    def test_user_registration(self):
        self.driver.get('https://accounts-qa.openstax.org/')
        assert('Sign in ' in self.driver.title), 'Unable to load page'
        self.driver.find_element(By.LINK_TEXT, 'Sign up').click()
        username = 'testuser_%s' % self.rword(5)
        first_name = 'Test User'
        last_name = self.rword(6)
        email = self.teacher.email
        self.wait.until(
            expect.visibility_of_element_located(
                (By.ID, 'register_username')
            )
        ).send_keys(username)
        self.driver.find_element(By.ID, 'register_email'). \
            send_keys(email)
        self.driver.find_element(By.ID, 'register_password'). \
            send_keys('password')
        self.driver.find_element(By.ID, 'register_password_confirmation'). \
            send_keys('password')
        self.driver.find_element(
            By.XPATH, '//input[@class="standard" and @name="commit"]'
        ).click()
        self.wait.until(
            expect.visibility_of_element_located(
                (By.LINK_TEXT, 'Continue')
            )
        ).click()
        self.wait.until(
            expect.visibility_of_element_located(
                (By.ID, 'register_username')
            )
        )
        self.driver.find_element(By.ID, 'register_title'). \
            send_keys(self.rword(2))
        self.driver.find_element(By.ID, 'register_first_name'). \
            send_keys(first_name)
        self.driver.find_element(By.ID, 'register_last_name'). \
            send_keys(last_name)
        self.driver.find_element(By.ID, 'register_suffix'). \
            send_keys(self.rword(2))
        self.driver.find_element(By.ID, 'register_full_name'). \
            send_keys('%s %s' % (first_name, last_name))
        self.driver.find_element(By.LINK_TEXT, 'Terms of Use').click()
        self.wait.until(
            expect.element_to_be_clickable(
                (By.XPATH, '//button[@class="close"]')
            )
        ).click()
        self.wait.until(
            expect.element_to_be_clickable(
                (By.LINK_TEXT, 'Privacy Policy')
            )
        ).click()
        self.wait.until(
            expect.element_to_be_clickable(
                (By.XPATH, '//button[@class="close"]')
            )
        ).click()
        i_agree = self.wait.until(
            expect.element_to_be_clickable(
                (By.ID, 'register_i_agree')
            )
        )
        submit = self.wait.until(
            expect.visibility_of_element_located(
                (By.ID, 'register_submit')
            )
        )
        if submit.get_attribute('class') != 'standard':
            i_agree.click()
        submit.click()
        assert('Your Account' in self.driver.title), 'Incorrect URL: %s' % \
            self.driver.current_url

    def test_user_login_standard(self):
        # resize the window to the standard HISD monitor width
        size = self.driver.get_window_size()
        assert(standard_window == (size['width'], size['height'])), \
            ('Window size set to: %sx%s, not %sx%s' %
             (size['width'], str(*size)))
        # open the test URL and click the login button
        self.driver.get(self.teacher.url)
        assert('OpenStax Tutor' in self.driver.title), 'Unable to load page'
        self.driver.find_element(By.LINK_TEXT, 'Login').click()
        # enter the username and password
        self.driver.find_element(By.ID, 'auth_key'). \
            send_keys(self.teacher.username)
        self.driver.find_element(By.ID, 'password'). \
            send_keys(self.teacher.password)
        # click on the sign in button to log in
        self.driver.find_element(
            By.XPATH, '//button[text()="Sign in"]'
        ).click()
        # look for the user drop down menu and click to open it
        self.wait.until(
            expect.presence_of_element_located(
                (By.CLASS_NAME, 'dropdown-toggle')
            )
        ).click()
        # is the log out link visible?
        logout = self.driver.find_element(By.CLASS_NAME, 'logout')
        assert('Log Out' in logout.text), 'Log out not visible'

    def test_user_login_compact(self):
        # resize the window to be closer to a mobile display width
        self.driver.set_window_size(*compressed_window)
        size = self.driver.get_window_size()
        assert(compressed_window == (size['width'], size['height'])), \
            ('Window size set to: ' + str(*size) +
             ', not ' + str(*compressed_window))
        # open the test URL and click the login button within the sub-menu
        self.driver.get(self.helper.user.url)
        assert('OpenStax Tutor' in self.driver.title), 'Unable to load page'
        user_menu = self.wait.until(
            expect.presence_of_element_located(
                (By.CLASS_NAME, 'navbar-toggle')
            )
        )
        assert(user_menu.is_displayed()), 'Menu not visible'
        user_menu.click()
        login = self.wait.until(
            expect.presence_of_element_located(
                (By.LINK_TEXT, 'Login')
            )
        )
        assert(login.is_displayed()), 'Login link not visible'
        login.click()
        # enter the username and password
        self.driver.find_element(By.ID, 'auth_key'). \
            send_keys(self.helper.user.username)
        self.driver.find_element(By.ID, 'password'). \
            send_keys(self.helper.user.password)
        # click on the sign in button to log in
        self.driver.find_element(
            By.XPATH, '//button[text()="Sign in"]').click()
        # look for the user drop down menu and click to open it
        user_menu = self.wait.until(
            expect.presence_of_element_located(
                (By.CLASS_NAME, 'navbar-toggle')
            )
        )
        assert(user_menu.is_displayed()), 'User menu not visible'
        user_menu.click()

    def test_accounts_login(self):
        # resize the window to the standard HISD monitor width
        size = self.driver.get_window_size()
        assert(standard_window == (size['width'], size['height'])), \
            ('Window size set to: ' + str(*size) +
             ', not ' + str(*standard_window))
        # open the test URL and click the login button within the sub-menu
        self.driver.get('https://accounts-qa.openstax.org/')
        assert('Sign in ' in self.driver.title), 'Unable to load page'
        # enter the username and password
        self.driver.find_element(By.ID, 'auth_key'). \
            send_keys(self.helper.user.username)
        self.driver.find_element(By.ID, 'password'). \
            send_keys(self.helper.user.password)
        # click on the sign in button to log in
        self.driver.find_element(
            By.XPATH, '//button[text()="Sign in"]').click()
        # is the logged in user's login visible?
        self.driver.find_element(
            By.LINK_TEXT, self.helper.user.username).click()

    def test_account_login_failure(self):
        # resize the window to the standard HISD monitor width
        size = self.driver.get_window_size()
        assert(standard_window == (size['width'], size['height'])), \
            ('Window size set to: ' + str(*size) +
             ', not ' + str(*standard_window))
        self.driver.get('https://accounts-qa.openstax.org/')
        assert('Sign in ' in self.driver.title), 'Unable to load page'
        self.driver.find_element(By.ID, 'auth_key'). \
            send_keys('not_a_user_94720475')
        self.driver.find_element(By.ID, 'password'). \
            send_keys('failed_password')
        self.driver.find_element(By.XPATH,
                                 '//button[text()="Sign in"]').click()
        error_message = self.driver.find_element(By.XPATH,
                                                 '//p//strong/parent::*')
        assert('Incorrect' in error_message.text)

    def test_course_select(self):
        # resize the window to the standard HISD monitor width
        size = self.driver.get_window_size()
        assert(standard_window == (size['width'], size['height'])), \
            ('Window size set to: ' + str(*size) +
             ', not ' + str(*standard_window))
        self.driver.get(self.helper.user.url)
        assert('OpenStax Tutor' in self.driver.title), 'Unable to load page'
        login = self.wait.until(
            expect.presence_of_element_located(
                (By.LINK_TEXT, 'Login')
            )
        )
        assert(login.is_displayed()), 'Login link not visible'
        login.click()
        self.driver.find_element(By.ID, 'auth_key'). \
            send_keys(self.helper.user.username)
        self.driver.find_element(By.ID, 'password'). \
            send_keys(self.helper.user.password)
        self.driver.find_element(By.XPATH, '//button[text()="Sign in"]'). \
            click()
        self.wait.until(
            expect.element_to_be_clickable(
                (By.XPATH, '//div[contains(@data-title, "Physics")]//a')
            )
        ).click()
        dashboard = self.wait.until(
            expect.element_to_be_clickable(
                (By.XPATH, '//a[@class="navbar-brand active"]')
            )
        )
        assert('Physics' in dashboard.text)

    def test_password_reset(self):
        # resize the window to the standard HISD monitor width
        size = self.driver.get_window_size()
        assert(standard_window == (size['width'], size['height'])), \
            ('Window size set to: ' + str(*size) +
             ', not ' + str(*standard_window))
        self.driver.set_window_size(*standard_window)
        self.driver.get('https://accounts-qa.openstax.org/')
        assert('Sign in ' in self.driver.title), 'Unable to load page'
        self.driver.find_element(By.LINK_TEXT, 'Forgot password?').click()
        self.wait.until(
            expect.visibility_of_element_located(
                (By.ID, 'forgot_password_username_or_email')
            )
        ).send_keys(self.helper.user.username)
        self.driver.find_element(By.NAME, 'commit').click()
        self.helper.sleep(4.5)
        try:
            self.driver.find_element(
                By.XPATH, '//p[contains(text(),"No verified email")]'
            )
            assert(False), 'No verified email - stubbed for now'
        except:
            pass
        reset_message = self.wait.until(
            expect.visibility_of_element_located(
                (By.XPATH, '//span[contains(@class,"ui-icon-info")]')
            )
        )
        reset_message = reset_message.find_element(By.XPATH, '..')
        assert('Password reset' in reset_message.text), 'Reset failed'
        body = self.driver.find_element(By.TAG_NAME, 'body')
        import platform
        if platform.system() == 'Darwin':  # Mac
            body.send_keys(Keys.COMMAND + 't')
        else:
            body.send_keys(Keys.CONTROL + 't')
        self.driver.get('https://mail.google.com/')
        assert('Gmail' in self.driver.title), 'Gmail login not available'
        username = self.driver.find_element(By.ID, 'Email')
        username.send_keys(self.teacher.email)
        next_button = self.driver.find_element(By.ID, 'next')
        next_button.click()
        password = self.driver.find_element(By.ID, 'Passwd')
        password.send_keys(self.teacher.email_password)
        stay_signed_in = self.driver.find_element(By.ID, 'PersistentCookie')
        if stay_signed_in.is_selected():
            stay_signed_in.click()
        next_button = self.driver.find_element(By.ID, 'signIn')
        next_button.click()
        try:
            reset_email = self.wait.until(
                expect.visibility_of_element_located(
                    (By.CLASS_NAME, 'y6')
                )
            )
        except:
            assert(False), 'Email message not received'
        reset_email.click()
        try:
            reset_link = self.wait.until(
                expect.presence_of_element_located(
                    (By.XPATH, '//a[contains(@href, "accounts-qa")]')
                )
            )
        except:
            assert(False), 'Wrong e-mail message selected'
        link = reset_link.get_attribute('href')
        self.driver.find_element(
            By.XPATH,
            '//a[@title="Gmail" and contains(@href,"inbox")]'
        ).click()
        reset_email = self.wait.until(
            expect.visibility_of_element_located(
                (By.CLASS_NAME, 'y6')
            )
        )
        trash_can = self.driver.find_element(By.LINK_TEXT, 'Trash')
        chain = ActionChains(self.driver). \
            move_to_element(reset_email). \
            drag_and_drop(reset_email, trash_can)
        chain.perform()
        self.driver.close()
        self.driver = StaxHelper.run_on(
            StaxHelper.LOCAL, self.ps, self.desired_capabilities
        )
        self.driver.implicitly_wait(15)
        self.driver.set_window_size(*standard_window)
        self.wait = WebDriverWait(self.driver, 15)
        self.driver.get(link)
        new_password = self.wait.until(
            expect.presence_of_element_located(
                (By.XPATH, '//input[@name="reset_password[password]"]')
            )
        )
        new_password.send_keys(self.teacher.password)
        repeat_password = self.driver.find_element(
            By.ID, 'reset_password_password_confirmation'
        )
        repeat_password.send_keys(self.teacher.password)
        submit = self.driver.find_element(By.NAME, 'commit')
        submit.click()
        reset_message = self.wait.until(
            expect.visibility_of_element_located(
                (By.CLASS_NAME, 'ui-icon-info')
            )
        )
        reset_message = reset_message.find_element(By.XPATH, '..')
        assert('reset successfully' in reset_message.text)

    def test_ost_logo_click_user_not_logged_in(self):
        url = 'https://tutor-qa.openstax.org/'
        anchor = '#home'
        self.driver.get(url)
        assert('OpenStax Tutor' in self.driver.title), 'Unable to load page'
        self.wait.until(
            expect.element_to_be_clickable(
                (By.XPATH,
                 '//a[@class="navbar-brand" and @href="%s"]' % anchor)
            )
        ).click()
        self.wait.until(
            expect.element_to_be_clickable(
                (By.XPATH,
                 '//a[@class="navbar-brand" and @href="%s"]' % anchor)
            )
        )
        full_url = url + anchor
        assert(self.driver.current_url == full_url), 'Not at dashboard'

    def test_ost_logo_click_user_logged_in(self):
        self.helper.user.login(self.driver,
                               self.helper.user.username,
                               self.helper.user.password)
        url = self.helper.user.url
        if url[-1:] == '/':
            url = url[:-1]
        route = '/dashboard/'
        assert('OpenStax Tutor' in self.driver.title), 'Unable to load page'
        self.wait.until(
            expect.element_to_be_clickable(
                (By.XPATH,
                 '//a[contains(@class,"navbar-brand") ' +
                 'and @href="%s"]' % route)
            )
        ).click()
        local_wait = WebDriverWait(self.driver, 5)
        try:
            local_wait.until(
                expect.element_to_be_clickable(
                    (By.CLASS_NAME, 'tutor-course-item')
                )
            )
        except:
            assert('calendar' in self.driver.current_url), 'Not at calendar'
        full_url = url + route
        assert(self.driver.current_url == full_url), 'Not at dashboard'

    def test_user_updates_profile_information(self):
        self.helper.user.login(self.driver,
                               self.helper.user.username,
                               self.helper.user.password,
                               'https://accounts-qa.openstax.org/')
        heading = self.wait.until(
            expect.visibility_of_element_located(
                (By.ID, 'page-heading')
            )
        )
        new_title = self.rword(3)
        new_suffix = self.rword(2)
        assert(heading.text == 'Your Account'), 'Not at the profile control'
        self.driver.find_element(By.ID, 'user_title').clear()
        self.driver.find_element(By.ID, 'user_title'). \
            send_keys(new_title)
        self.driver.find_element(By.ID, 'user_suffix').clear()
        self.driver.find_element(By.ID, 'user_suffix'). \
            send_keys(new_suffix)
        self.driver.find_element(
            By.XPATH, '//input[contains(@value,"Update Profile")]').click()
        self.wait.until(
            expect.visibility_of_element_located(
                (By.XPATH, '//span[contains(@class,"ui-icon")]')
            )
        )
        title = self.driver.find_element(By.ID, 'user_title')
        assert(title.get_attribute('value') == new_title), \
            'Change to the title failed'
        suffix = self.driver.find_element(By.ID, 'user_suffix')
        assert(suffix.get_attribute('value') == new_suffix), \
            'Change to the suffix failed'

    def test_user_email_management(self):
        self.helper.user.login(self.driver,
                               self.helper.user.username,
                               self.helper.user.password,
                               'https://accounts-qa.openstax.org/')
        heading = self.wait.until(
            expect.visibility_of_element_located(
                (By.ID, 'page-heading')
            )
        )
        assert(heading.text == 'Your Account'), 'Not at the profile control'
        print('At Account control')
        self.driver.find_element(By.LINK_TEXT, 'Manage Email Addresses'). \
            click()
        print('Retrieve e-mails')
        emails = self.driver.find_elements(By.XPATH,
                                           '//tr/td[contains(text(),"@")]')
        for email in emails:
            print(email.text)
            if self.teacher.email in email.text:
                print('Removing current e-mail to rerun the test')
                self.driver.find_element(
                    By.XPATH,
                    '//a[contains(@data-confirm,"%s")]' %
                    self.teacher.email
                ).click()
                self.driver.switch_to_alert().accept()
                print('E-mail cleared')
        self.driver.find_element(By.ID, 'contact_info_value'). \
            send_keys(self.teacher.email)
        self.driver.find_element(
            By.XPATH, '//input[@value="Add"]'
        ).click()
        print('Adding e-mail address')
        self.driver.get('https://mail.google.com/')
        assert('Gmail' in self.driver.title), 'Gmail login not available'
        print('Opening Gmail for confirmation e-mail')
        username = self.driver.find_element(By.ID, 'Email')
        username.send_keys(self.teacher.email)
        next_button = self.driver.find_element(By.ID, 'next')
        next_button.click()
        password = self.driver.find_element(By.ID, 'Passwd')
        password.send_keys(self.teacher.password)
        stay_signed_in = self.driver.find_element(By.ID, 'PersistentCookie')
        if stay_signed_in.is_selected():
            stay_signed_in.click()
        next_button = self.driver.find_element(By.ID, 'signIn')
        next_button.click()
        print('Signing in to Gmail')
        try:
            email_confirm = self.wait.until(
                expect.visibility_of_element_located(
                    (By.CLASS_NAME, 'y6')
                )
            )
        except Exception as ex:
            self.fail('Email message not received :: %s :: %s' %
                      (ex.__class__.__name__, ex))
        finally:
            try:
                email_confirm.click()
            except UnboundLocalError as ex:
                assert(False), 'Error logging into Google :: %s :: %s' % \
                    (ex.__class__.__name__, ex)
        print('Opening message')
        try:
            confirmation = self.wait.until(
                expect.presence_of_element_located(
                    (By.XPATH, '//a[contains(@href, "accounts-qa")]')
                )
            )
        except Exception as ex:
            self.fail('Wrong e-mail message selected :: %s :: %s' %
                      (ex.__class__.__name__, ex))
        link = confirmation.get_attribute('href')
        print('Save the confirmation URL')
        self.driver.close()
        self.driver = self.helper.run_on(self.ps, self.desired_capabilities)
        self.driver.implicitly_wait(15)
        self.driver.set_window_size(*standard_window)
        try:
            if self.wait:
                self.wait = None
        except:
            pass
        self.wait = WebDriverWait(self.driver, 15)
        self.driver.get(link)
        print('Open the confirmation URL')
        heading = self.wait.until(
            expect.visibility_of_element_located(
                (By.ID, 'page-heading')
            )
        )
        assert('Verification' in heading.text), 'Email not verified'
        print('E-mail verified')
        self.teacher.login(self.driver,
                           self.teacher.username,
                           self.teacher.password,
                           'https://accounts-qa.openstax.org/')
        self.wait.until(
            expect.visibility_of_element_located(
                (By.LINK_TEXT, 'Manage Email Addresses')
            )
        ).click()
        try:
            email = self.driver.find_elements(By.XPATH,
                                              '//td[contains(text(),"@")]')
        except Exception as ex:
            self.fail('Email not found in verified list :: %s :: %s' %
                      (ex.__class__.__name__, ex))



# a helper to use actions

import datetime
import inspect
import os
import sys
import unittest

from builtins import FileNotFoundError
from requests import HTTPError
from selenium import webdriver
from selenium.common.exceptions import WebDriverException
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as expect
from selenium.webdriver.support.ui import WebDriverWait
from time import sleep
from urllib.parse import urlparse, ParseResult

try:
    from staxing.assignment import Assignment
except ImportError:
    from assignment import Assignment


class StaxHelper(object):
    ''''''
    CONDENSED_WIDTH = 767  # pixels
    DEFAULT_WAIT_TIME = 15  # seconds

    def __init__(self, driver_type='chrome', capabilities=None,
                 pasta_user=None, wait_time=DEFAULT_WAIT_TIME,
                 opera_driver='/Applications/operadriver', initial_user=None):

        self.webdriver_service = None
        self.user = initial_user
        if driver_type == 'saucelabs' and pasta_user is None:
            raise TypeError('A Sauce Labs user is required for remote testing')
        self.pasta = pasta_user
        self.driver = self.run_on(driver_type, self.pasta, capabilities)
        self.driver.implicitly_wait(wait_time)

    def run_on(self, driver_type, pasta_user, capabilities=None,
               wait=DEFAULT_WAIT_TIME, opera_driver=''):
        try:
            return {
                'firefox': lambda: webdriver.Firefox(),
                'chrome': lambda: webdriver.Chrome(),
                'ie': lambda: webdriver.Ie(),
                'opera': lambda: self.start_opera(opera_driver),
                'phantomjs': lambda: webdriver.PhantomJS(),
                'saucelabs': lambda: webdriver.Remote(
                    command_executor=(
                        'http://%s:%s@ondemand.saucelabs.com:80/wd/hub' %
                        (pasta_user.get_user(), pasta_user.get_access_key())
                    ),
                    desired_capabilities=capabilities
                ),
            }[driver_type]()
        except WebDriverException as err:
            raise FileNotFoundError(err)
        except Exception as err:
            raise self.WebDriverTypeException(
                msg='Unknown WebDriver type: "%s"' % driver_type,
                err=err.__traceback__
            )

    def start_opera(self, location):
        from selenium.webdriver.chrome import service
        webdriver_service = service.Service(location)
        webdriver_service.start()
        return webdriver.Remote(
            webdriver_service.service_url,
            webdriver.DesiredCapabilities.OPERA
        )

    @classmethod
    def date_string(cls, day_delta=0, str_format='%m/%d/%Y'):
        return (datetime.date.today() + datetime.timedelta(days=day_delta)). \
            strftime(str_format)

    def quit(self):
        try:
            self.driver.quit()
        except:
            pass

    def get(self, url):
        self.driver.get(url)

    def get_window_size(self, dimension=None):
        get_size = self.driver.get_window_size()
        if dimension is None:
            return get_size
        if dimension not in get_size:
            raise IndexError('Unknown dimension: %s' % dimension)
        return get_size[dimension]

    def sleep(self, seconds=1):
        sleep(seconds)


class WebDriverTypeException(WebDriverException):
    def __init__(self, msg, err):
        self.msg = msg
        self.__traceback__ = err

    def __str__(self):
        return repr(self.msg, self.__traceback__)


class User(object):
    ''''''
    DEFAULT_WAIT_TIME = StaxHelper.DEFAULT_WAIT_TIME

    def __init__(self, username=None, password=None, site=None, email=None,
                 email_username=None, email_password=None):
        self.username = username
        self.password = password
        parse = list(urlparse(site if urlparse(site).scheme else
                     '%s%s' % ('//', site)))
        parse[0] = b'https'
        for index, value in enumerate(parse):
            parse[index] = value.decode('utf-8') if isinstance(value, bytes) \
                else value
        parse = ParseResult(*parse)
        self.url = parse.geturl()
        self.email = email
        self.email_username = email_username
        self.email_password = email_password

    def login(self, driver, username=None, password=None, url=None,
              wait_time=DEFAULT_WAIT_TIME):
        '''
        Tutor login control

        Requires a Tutor or Accounts instance
        Branching to deal with standard or compact screen widths
        '''
        wait = WebDriverWait(driver, wait_time)
        url_address = self.url if url is None else url
        # open the URL
        driver.get(url_address)
        if 'tutor' in url_address:
            # check to see if the screen width is normal or condensed
            if driver.get_window_size()['width'] <= \
                    StaxHelper.CONDENSED_WIDTH:
                # get small-window menu toggle
                is_collapsed = driver.find_element(
                    By.XPATH,
                    '//button[contains(@class,"navbar-toggle")]'
                )
                # check if the menu is collapsed and, if yes, open it
                if('collapsed' in is_collapsed.get_attribute('class')):
                    is_collapsed.click()
            driver.find_element(By.LINK_TEXT, 'Login').click()
        wait.until(
            expect.presence_of_element_located(
                (By.XPATH, '//html/body')
            )
        )
        import re
        src = driver.page_source
        text_located = re.search(r'openstax', src.lower()) is not None
        if not text_located:
            raise self.LoginError('Non-OpenStax URL: %s' %
                                  driver.current_url)
        # enter the username and password
        driver.find_element(By.ID, 'auth_key'). \
            send_keys(self.username if username is None else username)
        driver.find_element(By.ID, 'password'). \
            send_keys(self.password if password is None else password)
        # click on the sign in button
        driver.find_element(
            By.XPATH, '//button[text()="Sign in"]'
        ).click()

    def logout(self, driver):
        '''
        Logout control
        '''
        url_address = driver.current_url
        if 'tutor' in url_address:
            self.tutor_logout(driver)
        elif 'accounts' in url_address:
            self.accounts_logout(driver)
        else:
            raise HTTPError('Not an OpenStax URL')

    def open_user_menu(self, driver, wait_time=DEFAULT_WAIT_TIME):
        '''
        Hamburger menu opener

        ToDo: branching to handle if a toggle is already open
        '''
        wait = WebDriverWait(driver, wait_time)
        if driver.get_window_size('width') <= StaxHelper.CONDENSED_WIDTH:
            # compressed window display on Tutor
            wait.until(
                expect.visibility_of_element_located(
                    (By.CLASS_NAME, 'navbar-toggle')
                )
            ).click()
        wait.until(
            expect.visibility_of_element_located(
                (By.CLASS_NAME, 'dropdown-toggle')
            )
        ).click()

    def tutor_logout(self, driver, wait_time=DEFAULT_WAIT_TIME):
        '''
        Tutor logout helper
        '''
        self.open_user_menu(driver, wait_time)
        WebDriverWait(driver, wait_time).until(
            expect.visibility_of_element_located(
                (By.XPATH, '//button[@aria-label="Sign out"]')
            )
        ).click()

    def accounts_logout(self, driver):
        '''
        Accounts logout helper
        '''
        driver.find_element(By.LINK_TEXT, 'Sign out').click()

    def select_course(self, driver, title=None, category=None,
                      wait_time=DEFAULT_WAIT_TIME):
        '''
        Course selection

        ToDo: allow selection of course 3 or higher
        '''
        if 'dashboard' not in driver.current_url:
            return
        if title:
            uses_option = 'title'
            course = title
        elif category:
            uses_option = 'appearance'
            course = category.lower()
        else:
            raise self.LoginError('Unknown course selection "%s"' %
                                  title if title else category)
        WebDriverWait(driver, wait_time).until(
            expect.element_to_be_clickable(
                (
                    By.XPATH, '//div[@data-%s="%s"]//a' %
                    (uses_option, course)
                )
            )
        ).click()

    def view_reference_book(self, driver, wait_time=DEFAULT_WAIT_TIME):
        '''
        Access the reference book
        '''
        try:
            driver.find_element(
                By.XPATH, '//div/a[contains(@class,"view-reference-guide")]'
            ).click()
            return
        except:
            pass
        self.open_user_menu(driver, wait_time)
        driver.find_element(
            By.XPATH, '//li/a[contains(@class,"view-reference-guide")]'
        ).click()


class LoginError(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class Teacher(User):
    ''''''
    DEFAULT_WAIT_TIME = User.DEFAULT_WAIT_TIME

    def __init__(self, username=None, password=None, site=None, email=None,
                 use_env_vars=False):
        if use_env_vars:
            username = os.environ['TEACHER_USER']
            password = os.environ['TEACHER_PASSWORD']
            site = os.environ['SERVER_URL']
            email = os.environ['TEST_EMAIL_ACCOUNT']
            email_username = os.environ['TEST_EMAIL_USER']
            email_password = os.environ['TEST_EMAIL_PASSWORD']
        super().__init__(username, password, site, email, email_username,
                         email_password)

    def add_assignment(self, driver, assignment, args):
        '''
        Add an assignment
        '''
        assign = Assignment()
        assign.add[assignment](
            driver=driver,
            name=args['title'],
            description=args['description'],
            periods=args['periods'],
            state=args['status'],
            url=args['url'] if 'url' in args else None,
            reading_list=args['reading_list'] if 'reading_list' in args else
            None,
            problems=args['problems'] if 'problems' in args else None,
        )

    def change_assignment(self, driver, assignment, args):
        '''
        Alter an existing assignment
        '''
        assign = Assignment()
        assign.edit[assignment](
            driver=driver,
            name=args['title'],
            description=args['description'],
            periods=args['periods'],
            state=args['status'],
            url=args['url'] if 'url' in args else None,
            reading_list=args['reading_list'] if 'reading_list' in args else
            None,
            problems=args['problems'] if 'problems' in args else None,
        )

    def delete_assignment(self, driver, assignment, args):
        '''
        Delete an existing assignment (if available)
        '''
        assign = Assignment()
        assign.remove[assignment](
            driver=driver,
            name=args['title'],
            description=args['description'],
            periods=args['periods'],
            state=args['status'],
            url=args['url'] if 'url' in args else None,
            reading_list=args['reading_list'] if 'reading_list' in args else
            None,
            problems=args['problems'] if 'problems' in args else None,
        )

    def goto_menu_item(self, driver, item, wait_time=DEFAULT_WAIT_TIME):
        '''
        Go to a specific user menu item
        '''
        if 'courses' in driver.current_url:
            self.open_user_menu(driver)
            WebDriverWait(driver, wait_time).until(
                expect.element_to_be_clickable(
                    (By.LINK_TEXT, item)
                )
            ).click()

    def goto_calendar(self, driver):
        '''
        Return the teacher to the calendar dashboard
        '''
        try:
            return driver.find_element(
                By.XPATH, '//a[contains(@class,"navbar-brand")]'
            ).click()
        except:
            self.goto_menu_item('Dashboard')

    def goto_performance_forecast(self, driver, wait_time=DEFAULT_WAIT_TIME):
        '''
        Access the performance forecast page
        '''
        self.goto_menu_item(driver, 'Performance Forecast', wait_time)

    def goto_student_scores(self, driver, wait_time=DEFAULT_WAIT_TIME):
        '''
        Access the student scores page
        '''
        self.goto_menu_item(driver, 'Student Scores', wait_time)


class Student(User):
    ''''''
    DEFAULT_WAIT_TIME = User.DEFAULT_WAIT_TIME

    def __init__(self, username=None, password=None, site=None, email=None,
                 use_env_vars=False):
        if use_env_vars:
            username = os.environ['STUDENT_USER']
            password = os.environ['STUDENT_PASSWORD']
            site = os.environ['SERVER_URL']
            email = os.environ['TEST_EMAIL_ACCOUNT']
            email_username = os.environ['TEST_EMAIL_USER']
            email_password = os.environ['TEST_EMAIL_PASSWORD']
        super().__init__(username, password, site, email, email_username,
                         email_password)

    def work_assignment(self):
        '''
        Work an assignment

        ToDo: all
        '''
        raise NotImplementedError(inspect.currentframe().f_code.co_name)

    def goto_past_work(self):
        '''
        View work for previous weeks

        ToDo: all
        '''
        raise NotImplementedError(inspect.currentframe().f_code.co_name)

    def goto_performance_forecast(self):
        '''
        View the student performance forecast

        ToDo: all
        '''
        raise NotImplementedError(inspect.currentframe().f_code.co_name)

    def practice(self):
        '''
        Complete a set of 5 practice problems

        ToDo: all
        '''
        raise NotImplementedError(inspect.currentframe().f_code.co_name)


class Admin(User):
    ''''''
    DEFAULT_WAIT_TIME = User.DEFAULT_WAIT_TIME

    def __init__(self, username=None, password=None, site=None, email=None,
                 use_env_vars=False):
        if use_env_vars:
            username = os.environ['ADMIN_USER']
            password = os.environ['ADMIN_PASSWORD']
            site = os.environ['SERVER_URL']
            email = os.environ['TEST_EMAIL_ACCOUNT']
            email_username = os.environ['TEST_EMAIL_USER']
            email_password = os.environ['TEST_EMAIL_PASSWORD']
        super().__init__(username, password, site, email, email_username,
                         email_password)

    def goto_admin_control(self, driver, wait_time=WebDriverWait):
        '''
        Access the administrator controls
        '''
        wait = WebDriverWait(driver, wait_time)
        wait.until(
            expect.visibility_of_element_located(
                (
                    By.XPATH, '%s%s' %
                    ('//li[contains(@class,"-hamburger-menu")]/',
                     'a[@type="button"]')
                )
            )
        ).click()
        wait.until(
            expect.visibility_of_element_located(
                (By.LINK_TEXT, 'Admin')
            )
        ).click()
        wait.until(
            expect.visibility_of_element_located(
                (By.XPATH, '//h1[text()="Admin Console"]')
            )
        )

    def goto_courses(self, driver, wait_time=DEFAULT_WAIT_TIME):
        '''
        Access the course admin control
        '''
        wait = WebDriverWait(driver, wait_time)
        try:
            wait.until(
                expect.visibility_of_element_located(
                    (By.XPATH, '//h1[text()="Admin Console"]')
                )
            )
        except:
            self.goto_admin_control(driver, wait_time)
        organization = wait.until(
            expect.element_to_be_clickable(
                (By.PARTIAL_LINK_TEXT, 'Course Organization')
            )
        )
        if 'open' not in organization.find_element(By.XPATH, '..'). \
                get_attribute('class'):
            organization.click()
        wait.until(
            expect.element_to_be_clickable(
                (By.LINK_TEXT, 'Courses')
            )
        ).click()
        wait.until(
            expect.visibility_of_element_located(
                (By.XPATH, '//h1[text()="Courses"]')
            )
        )

    def goto_ecosystems(self, driver, wait_time=DEFAULT_WAIT_TIME):
        '''
        Access the ecosystem admin control
        '''
        wait = WebDriverWait(driver, wait_time)
        try:
            wait.until(
                expect.visibility_of_element_located(
                    (By.XPATH, '//h1[text()="Admin Console"]')
                )
            )
        except:
            self.goto_admin_control()
        content = wait.until(
            expect.element_to_be_clickable(
                (By.PARTIAL_LINK_TEXT, 'Content')
            )
        )
        if 'open' not in content.find_element(By.XPATH, '..'). \
                get_attribute('class'):
            content.click()
        wait.until(
            expect.element_to_be_clickable(
                (By.LINK_TEXT, 'Ecosystems')
            )
        ).click()
        wait.until(
            expect.visibility_of_element_located(
                (By.XPATH, '//h1[text()="Ecosystems"]')
            )
        )


class ContentQA(User):
    ''''''
    DEFAULT_WAIT_TIME = User.DEFAULT_WAIT_TIME

    def __init__(self, username=None, password=None, site=None, email=None,
                 use_env_vars=False):
        if use_env_vars:
            username = os.environ['CONTENT_USER']
            password = os.environ['CONTENT_PASSWORD']
            site = os.environ['SERVER_URL']
            email = os.environ['TEST_EMAIL_ACCOUNT']
            email_username = os.environ['TEST_EMAIL_USER']
            email_password = os.environ['TEST_EMAIL_PASSWORD']
        super().__init__(username, password, site, email, email_username,
                         email_password)

# ########################################################################### #


class TestStaxHelper(unittest.TestCase):
    def test_staxhelper_chrome(self):
        try:
            helper = StaxHelper().quit()
            helper = StaxHelper(initial_user=User())
            helper.quit()
            helper = StaxHelper(driver_type='chrome').quit()
        except FileNotFoundError or WebDriverException:
            print('\n> Chrome Driver not available <')
        finally:
            if bool(helper.driver):
                helper.driver.quit()

    def test_staxhelper_firefox(self):
        try:
            StaxHelper(driver_type='firefox').quit()
        except FileNotFoundError or WebDriverException:
            print('\n> FireFox Driver not available <')

    @unittest.skipIf(sys.platform == 'darwin', 'No IE on Mac')
    def test_staxhelper_ie(self):
        try:
            StaxHelper(driver_type='ie').quit()
        except FileNotFoundError or WebDriverException:
            print('\n> Internet Explorer Driver not available <')

    def test_staxhelper_opera(self):
        try:
            StaxHelper(driver_type='opera').quit()
        except FileNotFoundError or WebDriverException:
            print('\n> Opera Driver not available <')

    @unittest.skipIf(True, 'PhantomJS is not implemented')
    def test_staxhelper_phantomjs(self):
        try:
            StaxHelper(driver_type='phantomjs').quit()
        except FileNotFoundError or WebDriverException:
            print('\n> PhantomJS Driver not available <')

    def test_staxhelper_init_errs(self):
        with self.assertRaises(TypeError):
            StaxHelper(driver_type='saucelabs')
        with self.assertRaises(StaxHelper.WebDriverTypeException):
            StaxHelper(driver_type='no driver type')


class TestUser(unittest.TestCase):
    def setUp(self):
        self.user = User()
        self.helper = StaxHelper(initial_user=self.user)

    def tearDown(self):
        self.helper.quit()

    def test_user_object(self):
        assert(self.user.username is None), 'Username is set'
        assert(self.user.password is None), 'Password is set'
        assert(self.user.url is not None), 'URL is not parsed or is invalid'
        assert(self.user.email is None), 'E-mail is set'

    def test_user_login(self):
        self.user.login('student01', 'password',
                        'https://tutor-qa.openstax.org/')


class TestTeacherUser(unittest.TestCase):
    def setUp(self):
        self.user = Teacher()
        self.helper = StaxHelper(initial_user=self.user)

    def tearDown(self):
        self.helper.quit()


class TestStudentUser(unittest.TestCase):
    def setUp(self):
        self.user = Student()
        self.helper = StaxHelper(initial_user=self.user)

    def tearDown(self):
        self.helper.quit()


class TestAdminUser(unittest.TestCase):
    def setUp(self):
        self.user = Admin()
        self.helper = StaxHelper(initial_user=self.user)

    def tearDown(self):
        self.helper.quit()


class TestContentQAUser(unittest.TestCase):
    def setUp(self):
        self.user = ContentQA()
        self.helper = StaxHelper(initial_user=self.user)

    def teardown(self):
        self.helper.quit()


class TestAssignments(unittest.TestCase):
    def setUp(self):
        self.today = datetime.date.today()
        self.user = Teacher(helper=self.helper,
                            site='http://tutor-qa.openstax.org/',
                            username='physics',
                            password='password')
        self.helper = StaxHelper(initial_user=self.user)

    def tearDown(self):
        self.helper.quit()

    def test_assignments(self):
        begin = (self.today + datetime.timedelta(days=0)).strftime('%m/%d/%Y')
        end = (self.today + datetime.timedelta(days=3)).strftime('%m/%d/%Y')
        # start the code example
        self.user.login()
        self.user.select_course(category='physics')
        for chap in ['ch11', 'ch12']:
            reading = 'v6.131 chapter %s' % chap
            self.user.add_assignment(
                assignment='reading',
                args={
                    'title': reading,
                    'description': 'A diagnostic assignment for %s' % chap,
                    'periods': {'all': (begin, end)},
                    'reading_list': [chap],
                    'status': 'publish',
                }
            )
            sleep(5)
        homework = 'test-hw %s' % Assignment.rword(8)
        self.user.add_assignment(
            assignment='homework',
            args={
                'title': homework,
                'description': 'An auto-test assignment',
                'periods': {'all': (begin, end)},
                'problems': {'4': None,
                             '4.1': (4, 8),
                             '4.2': 'all',
                             '4.3': ['2102@1', '2104@1', '2175'],
                             'ch5': 5,
                             'tutor': 4},
                'status': 'draft',
            }
        )


if __name__ == "__main__":
    # execute only if run as a script
    unittest.main()



import unittest
import pytest

# from selenium.webdriver.common.by import By
# from selenium.webdriver.common.keys import Keys
# from selenium.webdriver.support import expected_conditions as expect
from selenium.webdriver.support.ui import WebDriverWait
from datetime import datetime

from pastasauce import PastaSauce, PastaDecorator
from . import StaxHelper, Student
try:
    from . import Assignment
except:
    from staxing.assignment import Assignment

NOT_STARTED = True

browsers = [{
    "platform": "Windows 7",
    "browserName": "firefox",
    "version": "40.0",
    "screenResolution": "1440x900"
}]

standard_window = (1440, 800)
compressed_window = (700, 500)


@PastaDecorator.on_platforms(browsers)
class TestTutorStudent(unittest.TestCase):
    ''''''
    def setUp(self):
        self.ps = PastaSauce()
        self.desired_capabilities['name'] = self.id()
        self.student = Student(use_env_vars=True)
        self.helper = StaxHelper(driver_type='chrome', pasta_user=self.ps,
                                 capabilities=self.desired_capabilities,
                                 initial_user=self.student)
        self.driver = self.helper.driver
        self.wait = WebDriverWait(self.driver, StaxHelper.DEFAULT_WAIT_TIME)
        self.driver.set_window_size(*standard_window)
        self.teacher.login()
        self.teacher.select_course(title='physics')
        self.rword = Assignment.rword
        self.screenshot_path = '/tmp/errors/'

    def tearDown(self):
        # Returns the info of exception being handled
        has_errors = self._test_has_failed()
        if has_errors:
            print(self.driver.current_url, '\n')
            date_and_time = datetime.now().strftime('%Y-%m-%d_%H-%M-%S-%f')
            filename = 'testerr_%s.png' % date_and_time
            self.driver.save_screenshot('%s%s' % (self.screenshot_path,
                                        filename))
        self.driver.quit()
        self.ps.update_job(self.driver.session_id, passed=has_errors)

    def _test_has_failed(self):
        # for 3.4. In 3.3, can just use self._outcomeForDoCleanups.success:
        for method, error in self._outcome.errors:
            if error:
                return True
        return False

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_student_views_dashboard(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_student_views_all_past_work(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_student_views_reference_book(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_student_works_a_standard_reading(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_student_works_an_intro_reading(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_student_works_a_homework(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_student_practices_weakest_topics_from_dashboard(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_student_practices_specific_topic_from_dashboard(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_student_views_complete_performance_forecast(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_student_practices_all_topics_from_forecast(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_student_practices_one_chapter_from_forecast(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_student_practices_specific_topic_from_forecast(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_student_works_an_external_assignment(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_student_reviews_a_finished_reading(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_student_reviews_a_finished_homework(self):
        ''''''



# import os
import datetime
# import inspect
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as expect
from selenium.webdriver.support.ui import WebDriverWait
# from requests import HTTPError

if __name__ == '__main__':
    from assignment import Assignment
else:
    from staxing.assignment import Assignment


class StaxHelper(object):
    ''''''
    LOCAL = False  # use ChromeDriver locally
    REMOTE = not LOCAL  # use Sauce Labs
    CONDENSED_WIDTH = 767  # pixels
    WAIT_TIME = Assignment.WAIT_TIME  # seconds



"""Tutor teacher test cases."""

import unittest
import pytest

from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as expect
from selenium.webdriver.support.ui import WebDriverWait
from datetime import datetime, date

from pastasauce import PastaSauce, PastaDecorator
from . import StaxHelper, Teacher

try:
    from . import Assignment
except:
    from staxing.assignment import Assignment

NOT_STARTED = True

browsers = [{
    "platform": "Windows 7",
    "browserName": "firefox",
    "version": "40.0",
    "screenResolution": "1440x900"
}]

standard_window = (1440, 800)
compressed_window = (700, 500)


@PastaDecorator.on_platforms(browsers)
class TestTutorTeacher(unittest.TestCase):
    """Teacher test cases."""

    def setUp(self):
        """Test case setup routine."""
        self.ps = PastaSauce()
        self.desired_capabilities['name'] = self.id()
        self.teacher = Teacher(use_env_vars=True)
        self.helper = StaxHelper(driver_type='saucelabs', pasta_user=self.ps,
                                 capabilities=self.desired_capabilities,
                                 initial_user=self.teacher)
        self.driver = self.helper.driver
        self.wait = WebDriverWait(self.driver, StaxHelper.DEFAULT_WAIT_TIME)
        self.driver.set_window_size(*standard_window)
        self.teacher.login(self.driver)
        self.teacher.select_course(self.driver, category='physics')
        self.rword = Assignment.rword
        self.screenshot_path = '/tmp/errors/'

    def tearDown(self):
        """Test case cleanup and reporting."""
        session = self.driver.session_id
        # Returns the info of exception being handled
        has_errors = self._test_has_failed()
        if has_errors:
            print(self.driver.current_url, '\n')
            date_and_time = datetime.now().strftime('%Y-%m-%d_%H-%M-%S-%f')
            filename = 'testerr_%s.png' % date_and_time
            self.driver.save_screenshot('%s%s' % (self.screenshot_path,
                                        filename))
        try:
            self.driver = None
            self.helper.driver.quit()
        except AttributeError:
            pass
        self.ps.update_job(session, passed=not has_errors)

    def _test_has_failed(self):
        # for 3.4. In 3.3, can just use self._outcomeForDoCleanups.success:
        for method, error in self._outcome.errors:
            if error:
                return True
        return False

    def test_teacher_views_calendar(self):
        ''''''
        today = date.today()
        today = today.strftime('%B %Y')
        cal_date = self.wait.until(
            expect.presence_of_element_located(
                (By.XPATH,
                 '//div[contains(@class,"calendar-header-label")]' +
                 '/span')
            )
        ).text
        assert(cal_date == today), 'Calendar date is not %s' % today

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_teacher_views_student_scores(self):
        ''''''
        # click on student scores
        self.helper.user.goto_student_scores(self.driver)

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_teacher_views_reference_book(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_teacher_views_class_roster(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_teacher_removes_a_student_from_class(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_teacher_moves_a_student_between_periods(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_teacher_views_reading_analytics_aggregate(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_teacher_views_performance_forecast_aggregate(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_teacher_views_single_student_performance_forecast(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_teacher_views_external_summary(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_teacher_views_homework_summary(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_teacher_views_reading_summary(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_teacher_views_event_summary(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_teacher_views_review_summary(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_teacher_views_single_student_homework(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_teacher_views_class_homework_details(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_teacher_views_single_student_reading(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_teacher_views_class_reading_details(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_teacher_views_single_student_review(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_teacher_views_class_review_details(self):
        ''''''

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_teacher_export_matches_student_scores(self):
        ''''''



from .staxing import Stax
from .helpers import StaxHelper, Admin, Student, Teacher
from .assignment import Assignment

if __name__ == '__main__':
    a = Stax
    b = StaxHelper
    c = Assignment
    d = Admin
    e = Student
    f = Teacher



# import regex

from unittest import TestCase

try:
    from helpers import StaxHelper
except:
    from staxing.helpers import StaxHelper


class TestHelper(TestCase):
    ''''''
    def setUp(self):
        self.helper = StaxHelper()

    def tearDown(self):
        self.helper.driver = None

    def test_run_on(self):
        ''''''  # driver = self.helper



import unittest
import pytest
import inspect

# from selenium.webdriver.common.by import By
# from selenium.webdriver.common.keys import Keys
# from selenium.webdriver.support import expected_conditions as expect
from selenium.webdriver.support.ui import WebDriverWait
from datetime import datetime

from pastasauce import PastaSauce, PastaDecorator
from . import StaxHelper, Teacher
try:
    from . import Assignment
except:
    from staxing.assignment import Assignment

browsers = [{
    "platform": "Windows 7",
    "browserName": "firefox",
    "version": "40.0",
    "screenResolution": "1440x900"
}]

standard_window = (1440, 800)
compressed_window = (700, 500)

NOT_STARTED = True


@PastaDecorator.on_platforms(browsers)
class TestTutorBasicAssignments(unittest.TestCase):
    ''''''
    def setUp(self):
        self.ps = PastaSauce()
        self.desired_capabilities['name'] = self.id()
        self.teacher = Teacher(use_env_vars=True)
        self.helper = StaxHelper(driver_type='chrome', pasta_user=self.ps,
                                 capabilities=self.desired_capabilities,
                                 initial_user=self.teacher)
        self.driver = self.helper.driver
        self.wait = WebDriverWait(self.driver, StaxHelper.DEFAULT_WAIT_TIME)
        self.driver.set_window_size(*standard_window)
        self.teacher.login(self.driver)
        self.teacher.select_course(self.driver, title='physics')
        self.rword = Assignment.rword
        self.screenshot_path = '/tmp/errors/'

    def tearDown(self):
        # Returns the info of exception being handled
        has_errors = self._test_has_failed()
        if has_errors:
            print(self.driver.current_url, '\n')
            date_and_time = datetime.now().strftime('%Y-%m-%d_%H-%M-%S-%f')
            filename = 'testerr_%s.png' % date_and_time
            self.driver.save_screenshot('%s%s' % (self.screenshot_path,
                                        filename))
        self.driver = None
        self.helper.quit()
        self.ps.update_job(self.driver.session_id, passed=has_errors)

    def _test_has_failed(self):
        # for 3.4. In 3.3, can just use self._outcomeForDoCleanups.success:
        for method, error in self._outcome.errors:
            if error:
                return True
        return False

    # @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_today_all_cancel(self):
        '''
        '''
        # all / 0 / 1 / 1S / Cancel
        print(inspect.currentframe().f_code.co_name)
        print(self.__class__.__name__)
        name = 'Automated 1'
        description = 'by period cancel'
        open_all = StaxHelper.date_string()
        due_all = StaxHelper.date_string(1)
        try:
            self.assign.add_new_reading(
                driver=self.driver,
                title=name,
                description=description,
                periods={
                    'all': (open_all, due_all), },
                readings=['1.1'],
                status=self.assign.CANCEL)
        except Exception as ex:
            self.fail('1: Add :: %s :: %s' % (ex.__class__.__name__, ex))

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_today_all_draft_and_delete(self):
        '''
        '''
        # 1st / 0 / 1 / 1S / Cancel
        # 2nd / 1 / 2 / 1S
        print(inspect.currentframe().f_code.co_name)
        print(self.__class__.__name__)
        name = 'Automated 2'
        description = 'by period cancel'
        open_p1 = StaxHelper.date_string()
        due_p1 = StaxHelper.date_string(1)
        open_p2 = StaxHelper.date_string(1)
        due_p2 = StaxHelper.date_string(2)
        try:
            self.assign.add_new_reading(
                driver=self.driver,
                title=name,
                description=description,
                periods={
                    '1st': (open_p1, due_p1),
                    '2nd': (open_p2, due_p2), },
                readings=['1.1'],
                status=self.assign.CANCEL)
        except Exception as ex:
            self.fail('1: Add :: %s :: %s' % (ex.__class__.__name__, ex))

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_today_all_publish(self):
        '''
        '''
        # 1st / 0 / 1 / 1S / Cancel
        # 2nd / 1 / 2 / 1S
        print(inspect.currentframe().f_code.co_name)
        print(self.__class__.__name__)
        name = 'Automated 3'
        description = 'by period cancel'
        open_p1 = StaxHelper.date_string()
        due_p1 = StaxHelper.date_string(1)
        open_p2 = StaxHelper.date_string(1)
        due_p2 = StaxHelper.date_string(2)
        try:
            self.assign.add_new_reading(
                driver=self.driver,
                title=name,
                description=description,
                periods={
                    '1st': (open_p1, due_p1),
                    '2nd': (open_p2, due_p2), },
                readings=['1.1'],
                status=self.assign.CANCEL)
        except Exception as ex:
            self.fail('1: Add :: %s :: %s' % (ex.__class__.__name__, ex))

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_today_by_period_cancel(self):
        '''
        '''
        # 1st / 0 / 1 / 1S / Cancel
        # 2nd / 1 / 2 / 1S
        print(inspect.currentframe().f_code.co_name)
        print(self.__class__.__name__)
        name = 'Automated 4'
        description = 'by period cancel'
        open_p1 = StaxHelper.date_string()
        due_p1 = StaxHelper.date_string(1)
        open_p2 = StaxHelper.date_string(1)
        due_p2 = StaxHelper.date_string(2)
        try:
            self.assign.add_new_reading(
                driver=self.driver,
                title=name,
                description=description,
                periods={
                    '1st': (open_p1, due_p1),
                    '2nd': (open_p2, due_p2), },
                readings=['1.1'],
                status=self.assign.CANCEL)
        except Exception as ex:
            self.fail('1: Add :: %s :: %s' % (ex.__class__.__name__, ex))

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_today_by_period_draft_and_delete(self):
        '''
        '''
        # 1st / 0 / 1 / 1S / Cancel
        # 2nd / 1 / 2 / 1S
        print(inspect.currentframe().f_code.co_name)
        print(self.__class__.__name__)
        name = 'Automated 5'
        description = 'by period cancel'
        open_p1 = StaxHelper.date_string()
        due_p1 = StaxHelper.date_string(1)
        open_p2 = StaxHelper.date_string(1)
        due_p2 = StaxHelper.date_string(2)
        try:
            self.assign.add_new_reading(
                driver=self.driver,
                title=name,
                description=description,
                periods={
                    '1st': (open_p1, due_p1),
                    '2nd': (open_p2, due_p2), },
                readings=['1.1'],
                status=self.assign.CANCEL)
        except Exception as ex:
            self.fail('1: Add :: %s :: %s' % (ex.__class__.__name__, ex))

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_today_by_period_publish(self):
        '''
        '''
        # 1st / 0 / 1 / 1S / Cancel
        # 2nd / 1 / 2 / 1S
        print(inspect.currentframe().f_code.co_name)
        print(self.__class__.__name__)
        name = 'Automated 6'
        description = 'by period cancel'
        open_p1 = StaxHelper.date_string()
        due_p1 = StaxHelper.date_string(1)
        open_p2 = StaxHelper.date_string(1)
        due_p2 = StaxHelper.date_string(2)
        try:
            self.assign.add_new_reading(
                driver=self.driver,
                title=name,
                description=description,
                periods={
                    '1st': (open_p1, due_p1),
                    '2nd': (open_p2, due_p2), },
                readings=['1.1'],
                status=self.assign.CANCEL)
        except Exception as ex:
            self.fail('1: Add :: %s :: %s' % (ex.__class__.__name__, ex))

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_future_all_cancel(self):
        '''
        '''
        # 1st / 0 / 1 / 1S / Cancel
        # 2nd / 1 / 2 / 1S
        print(inspect.currentframe().f_code.co_name)
        print(self.__class__.__name__)
        name = 'Automated 7'
        description = 'by period cancel'
        open_p1 = StaxHelper.date_string()
        due_p1 = StaxHelper.date_string(1)
        open_p2 = StaxHelper.date_string(1)
        due_p2 = StaxHelper.date_string(2)
        try:
            self.assign.add_new_reading(
                driver=self.driver,
                title=name,
                description=description,
                periods={
                    '1st': (open_p1, due_p1),
                    '2nd': (open_p2, due_p2), },
                readings=['1.1'],
                status=self.assign.CANCEL)
        except Exception as ex:
            self.fail('1: Add :: %s :: %s' % (ex.__class__.__name__, ex))

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_future_all_draft_and_delete(self):
        '''
        '''
        # 1st / 0 / 1 / 1S / Cancel
        # 2nd / 1 / 2 / 1S
        print(inspect.currentframe().f_code.co_name)
        print(self.__class__.__name__)
        name = 'Automated 8'
        description = 'by period cancel'
        open_p1 = StaxHelper.date_string()
        due_p1 = StaxHelper.date_string(1)
        open_p2 = StaxHelper.date_string(1)
        due_p2 = StaxHelper.date_string(2)
        try:
            self.assign.add_new_reading(
                driver=self.driver,
                title=name,
                description=description,
                periods={
                    '1st': (open_p1, due_p1),
                    '2nd': (open_p2, due_p2), },
                readings=['1.1'],
                status=self.assign.CANCEL)
        except Exception as ex:
            self.fail('1: Add :: %s :: %s' % (ex.__class__.__name__, ex))

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_future_all_publish_and_delete(self):
        '''
        '''
        # 1st / 0 / 1 / 1S / Cancel
        # 2nd / 1 / 2 / 1S
        print(inspect.currentframe().f_code.co_name)
        print(self.__class__.__name__)
        name = 'Automated 9'
        description = 'by period cancel'
        open_p1 = StaxHelper.date_string()
        due_p1 = StaxHelper.date_string(1)
        open_p2 = StaxHelper.date_string(1)
        due_p2 = StaxHelper.date_string(2)
        try:
            self.assign.add_new_reading(
                driver=self.driver,
                title=name,
                description=description,
                periods={
                    '1st': (open_p1, due_p1),
                    '2nd': (open_p2, due_p2), },
                readings=['1.1'],
                status=self.assign.CANCEL)
        except Exception as ex:
            self.fail('1: Add :: %s :: %s' % (ex.__class__.__name__, ex))

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_future_by_period_cancel(self):
        '''
        '''
        # 1st / 0 / 1 / 1S / Cancel
        # 2nd / 1 / 2 / 1S
        print(inspect.currentframe().f_code.co_name)
        print(self.__class__.__name__)
        name = 'Automated 10'
        description = 'by period cancel'
        open_p1 = StaxHelper.date_string()
        due_p1 = StaxHelper.date_string(1)
        open_p2 = StaxHelper.date_string(1)
        due_p2 = StaxHelper.date_string(2)
        try:
            self.assign.add_new_reading(
                driver=self.driver,
                title=name,
                description=description,
                periods={
                    '1st': (open_p1, due_p1),
                    '2nd': (open_p2, due_p2), },
                readings=['1.1'],
                status=self.assign.CANCEL)
        except Exception as ex:
            self.fail('1: Add :: %s :: %s' % (ex.__class__.__name__, ex))

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_future_by_period_draft_and_delete(self):
        '''
        '''
        # 1st / 0 / 1 / 1S / Cancel
        # 2nd / 1 / 2 / 1S
        print(inspect.currentframe().f_code.co_name)
        print(self.__class__.__name__)
        name = 'Automated 11'
        description = 'by period cancel'
        open_p1 = StaxHelper.date_string()
        due_p1 = StaxHelper.date_string(1)
        open_p2 = StaxHelper.date_string(1)
        due_p2 = StaxHelper.date_string(2)
        try:
            self.assign.add_new_reading(
                driver=self.driver,
                title=name,
                description=description,
                periods={
                    '1st': (open_p1, due_p1),
                    '2nd': (open_p2, due_p2), },
                readings=['1.1'],
                status=self.assign.CANCEL)
        except Exception as ex:
            self.fail('1: Add :: %s :: %s' % (ex.__class__.__name__, ex))

    @pytest.mark.skipif(NOT_STARTED, reason='Not started')
    def test_assignment_future_by_period_publish_and_delete(self):
        '''
        '''
        # 1st / 0 / 1 / 1S / Cancel
        # 2nd / 1 / 2 / 1S
        print(inspect.currentframe().f_code.co_name)
        print(self.__class__.__name__)
        name = 'Automated 12'
        description = 'by period cancel'
        open_p1 = StaxHelper.date_string()
        due_p1 = StaxHelper.date_string(1)
        open_p2 = StaxHelper.date_string(1)
        due_p2 = StaxHelper.date_string(2)
        try:
            self.assign.add_new_reading(
                driver=self.driver,
                title=name,
                description=description,
                periods={
                    '1st': (open_p1, due_p1),
                    '2nd': (open_p2, due_p2), },
                readings=['1.1'],
                status=self.assign.CANCEL)
        except Exception as ex:
            self.fail('1: Add :: %s :: %s' % (ex.__class__.__name__, ex))
